# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: FlexibleDocument.py,v 1.19 2003/06/12 09:17:32 fermigier Exp $

"""
  Flexible Document
"""

from zLOG import LOG, DEBUG

from random import randrange
from Globals import InitializeClass
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent, aq_inner

from OFS.Folder import Folder

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

from Products.NuxCPSDocuments.BaseDocument import BaseDocument, BaseDocument_adder
from Products.NuxCPSDocuments.FileDocument import FileDocument

from Products.CMFDefault.NewsItem import NewsItem

factory_type_information = (
    {'id': 'Flexible Document',
     'title': '_portal_type_Flexible Document',
     'content_icon': 'document_icon.gif',
     'product': 'NuxCPSFlexibleDocument',
     'factory': 'addFlexibleDocument',
     'meta_type': 'Flexible Document',
     'immediate_view': 'flexibledocument_edit_form',
     'allow_discussion': 1,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'flexibledocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'flexibledocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'flexibledocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'content_view',
                  'name': 'content_view',
                  'action': 'flexibledocument_content_view',
                  'visible': 0,
                  'permissions': (View, )},
                 {'id': 'isdocument',
                  'name': 'isdocument',
                  'action': 'isdocument',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'issearchabledocument',
                  'name': 'issearchabledocument',
                  'action': 'issearchabledocument',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )




class FlexibleDocument(BaseDocument):
    """
    Flexible Document
    """
    meta_type = 'Flexible Document'

    portal_type = 'Flexible Document'

    security = ClassSecurityInfo()

    blocks = ()

    _locked = 0

    def _getBlockTypes(self):
        return self.get_block_types()

    def _getSlotTypes(self):
        return self.get_slot_types()

    security.declareProtected(ModifyPortalContent, 'addFlexibleBlock')
    def addFlexibleBlock(self, block_type, before_block=None):
        """Adds a block to this flexible document"""
        if self.isDocumentLayoutLocked():
            return None
        block_id = 'block_%s' % (str(int(DateTime()))+str(randrange(1000,10000)), )
        block = Folder()
        block.id = block_id
        block.manage_addProperty('block_type', block_type, 'text')
        self._setObject(block_id, block)
        if before_block is not None:
            position = self.get_object_position(before_block)
            self.move_object_to_position(block_id, position)
        return block_id

    security.declareProtected(View, 'getRenderedBlocks')
    def getRenderedBlocks(self, render_type='renderer', block_id=None):
        """Returns a list of the rendered blocks"""
        mcat = getToolByName(self, 'portal_messages')
        blocks = self.objectIds('Folder')
        if block_id is not None:
            if block_id in blocks:
                blocks = (block_id, )
            else:
                blocks = ()
        rendered_blocks = []
        block_types = self._getBlockTypes()
        slot_types = self._getSlotTypes()
        slot_count = {}
        block_count = {}
        for block_id in blocks:
            if block_id.startswith('block_'):
                
                block = getattr(self, block_id)
                block_type = block.block_type
                
                block_nb = block_count.get(block_type, 0)
                block_count[block_type] = block_nb = block_nb + 1
                
                type_desc = block_types.get(block_type)
                if type_desc is None:
                    rendered_blocks.append('<p>Block type %s not supported</p>'
                                           % block_type)
                    continue
                reset_blocks = type_desc.get('reset_blocks', ())
                for reset_block in reset_blocks:
                    block_count[reset_block] = 0
                block_slots = block.objectIds()
                rendered_slots = {}
                for slot in type_desc['slots']:
                    slot_type = slot['type']
                    count = slot_count.get(slot_type, 0)
                    slot_count[slot_type] = count + 1
                    slot_id = slot['id']
                    slot_renderer = slot_types[slot_type][render_type]
                    slot_args = slot['args']
                    if slot_id not in block_slots:
                        slot_obj = None
                    else:
                        slot_obj = getattr(block, slot_id)
                    # XXX restricted traverse
                    meth = getattr(self, slot_renderer)
                    prefix = '%s_%s' % (block_id, slot_id)
                    rendered_slots[slot_id] = \
                        meth(block=block, object=slot_obj, count=count,
                            prefix=prefix, mcat=mcat, **slot_args).strip()
                block_renderer = type_desc['template']
                # XXX restricted traverse
                meth = getattr(self, block_renderer)
                rendered_blocks.append(
                    {
                        'block_id': block_id,
                        'block_type': block_type,
                        'content': meth(
                            block=block, count=block_nb,
                            slots=rendered_slots, mcat=mcat,
                            render_type=render_type,
                            block_count=block_count,
                        )
                    }
                )
        return rendered_blocks

    def edit(self, **kw):
        """Edit the document"""
        redirect_blocks = {}
        blocks = {}
        moves = []
        deletes = []
        change_types = {}
        locked = self.isDocumentLayoutLocked()
        for key, value in kw.items():
            if key.startswith('block_'):
                parts = key.split('_', 3)
                if len(parts)<4:
                    continue
                block_id = 'block_%s' % (parts[1], )
                redirect_blocks[block_id] = None
                slot_id = parts[2]
                arg_id = parts[3]
                block_args = blocks.get(block_id)
                if block_args is None:
                    blocks[block_id] = block_args = {}
                slot_args = block_args.get(slot_id)
                if slot_args is None:
                    block_args[slot_id] = slot_args = {}
                slot_args[arg_id] = value
            if not locked:
                if key.startswith('move_'):
                    parts = key.split('_', 2)
                    block_id = parts[2]
                    redirect_blocks[block_id] = None
                    moves.append(
                        {
                            'direction': parts[1],
                            'block_id': block_id,
                        }
                    )
                if key.startswith('delete_'):
                    deletes.append({'block_id': key[7:]})
                if key.startswith('change_type_'):
                    change_types[key[12:]] = value

        # modify blocks
        block_ids = self.objectIds('Folder')
        block_types = self._getBlockTypes()
        slot_types = self._getSlotTypes()
        for block_id, slot_args in blocks.items():
            if block_id in block_ids:
                block = getattr(self, block_id)
                block_type = block.block_type
                type_desc = block_types[block_type]
                block_slots = block.objectIds()
                for slot in type_desc['slots']:
                    slot_id = slot['id']
                    args = slot_args.get(slot_id)
                    if args is None:
                        continue
                    
                    if slot_id in block_slots:
                        slot_obj = getattr(block, slot_id)
                    else:
                        slot_obj = None
                    slot_type = slot['type']
                    slot_editor = slot_types[slot_type]['edit']
                    # XXX restricted traverse
                    meth = getattr(self, slot_editor)
                    meth(block=block, object=slot_obj, slot_id=slot_id, **args)

        # move blocks
        for move in moves:
            direction = move['direction']
            block_id = move['block_id']
            if direction == 'up':
                # XXX should only move 'block_*'
                self.move_object_up(block_id)
            else:
                self.move_object_down(block_id)

        # change types
        for key, value in change_types.items():
            if key in block_ids:
                block = getattr(self, key)
                block.block_type = value

        # delete blocks
        for delete in deletes:
            block_id = delete['block_id']
            if redirect_blocks.has_key(block_id):
                del redirect_blocks[block_id]
            self._delObject(block_id)

        BaseDocument.edit(self, **kw)
        if redirect_blocks.keys():
            return redirect_blocks.keys()[0]
        
        

    security.declareProtected(ModifyPortalContent, 'lockDocumentLayout')
    def lockDocumentLayout(self):
        """locks this document layout"""
        self._locked = 1

    security.declareProtected(ModifyPortalContent, 'isDocumentLayoutLocked')
    def isDocumentLayoutLocked(self):
        """returns wether this document layout is locked"""
        return self._locked

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        blocks = self.objectIds('Folder')
        text = ''
        block_types = self._getBlockTypes()
        slot_types = self._getSlotTypes()
        for block_id in blocks:
            if block_id.startswith('block_'):
                block = getattr(self, block_id)
                block_type = block.block_type
                type_desc = block_types.get(block_type)
                if type_desc is None:
                    continue
                block_slots = block.objectIds()
                for slot in type_desc['slots']:
                    slot_type = slot['type']
                    slot_id = slot['id']
                    slot_indexer = slot_types[slot_type].get('index')
                    slot_args = slot['args']
                    if slot_id not in block_slots:
                        slot_obj = None
                    else:
                        slot_obj = getattr(block, slot_id)
                    if slot_indexer is None or slot_obj is None:
                        continue
                    # XXX restricted traverse
                    meth = getattr(self, slot_indexer)
                    text = text + ' ' + str(
                        meth(
                            block=block, object=slot_obj,
                            slot_id=slot_id, **slot_args
                        )
                    )
        return '%s %s' % (BaseDocument.SearchableText(self), text)
    
    
    def setPortalType(self, portal_type):
        """
        Set Portal Type to improve search
        """
        self.portal_type = portal_type
        self.meta_type = portal_type
        
    security.declareProtected(View, 'getSlot')
    def getSlot(self, slot_id, block_id):
        """ 
        Get a selected slot in a selected block 
        """
        block = getattr(self, block_id, None)
        if(block is not None):
            slot = getattr(block, slot_id, None)
        else:
            slot = None    
            
        return slot
    
    def start(self):
        """ Return our start time as a string. """
        date = self.created()
        
        for block in self.objectValues('Folder'):
            slot = getattr(block, 'event', None)
            if(slot is not None and slot.text is not ''):
                date = DateTime(slot.text)             
        
        return date
          
    def end(self):
        """ Return our end time as a string. """
        return self.start()
    

NewsItem.start = FlexibleDocument.start
    
    
InitializeClass(FlexibleDocument)


def addFlexibleDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a Flexible Document."""
    ob = FlexibleDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)
