# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Author: Florent Guillaume <mailto:fg@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: BaseHierarchy.py,v 1.5 2003/04/06 09:42:38 fermigier Exp $

"""
  Base Hierarchy
"""

__version__ = '$Revision: 1.5 $'[11:-2]

from zLOG import LOG, DEBUG
import string, copy
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base

from Products.CMFCore.CMFCorePermissions import setDefaultRoles, \
     AccessContentsInformation, ManagePortal
from Products.CMFCore.utils import SimpleItemWithProperties, getToolByName


from HierarchiesTool import RebuildHierarchy, addHierarchyFactory


class BaseHierarchy(SimpleItemWithProperties):
    """
    Base Class for Hierarchy
    """
    meta_type = None
    HierarchyLevelType = None
    criterion = None

    security = ClassSecurityInfo()

    _properties = (
        {'id': 'title', 'type': 'string', 'mode': 'w', 
            'label': 'Title'},
        {'id': 'root', 'type': 'string', 'mode': 'w', 
            'label': 'Hierarchy root in portal'},
        {'id': 'skiproot', 'type': 'boolean', 'mode': 'w', 
            'label': 'Skip the root'},
    )

    title = ''
    root = ''
    skiproot = 0

    hierlist = []
    hierdict = {}
    hierkeys = {}

    def __init__(self, id):
        self.id = id

    security.declarePublic('getCriterion')
    def getCriterion(self):
        """Get the criterion for this hierarchy."""
        return self.criterion

    security.declarePublic('getRootObject')
    def getRootObject(self):
        """Get the root object for this hierarchy."""
        root = self.root
        relativepath = tuple(filter(None, string.split(root, '/')))
        portalpath = getToolByName(self, 'portal_url').getPortalObject().getPhysicalPath()
        rootpath = portalpath + relativepath
        absroot = string.join(rootpath, '/')
        try:
            root_obj = self.restrictedTraverse(absroot) # may raise an exception
        except KeyError:
            raise KeyError, 'The hierarchy root path is probably incorrect'
        return root_obj

    security.declarePrivate('_recursive_build')
    def _recursive_build(self, obj, depth, parentpath, hierdict, order):
        # obj is a Hierarchy Level at "depth". Add its data and recurse.
        dict = {} # the content of this dict must not be mutable XXX ?
        dict['id'] = obj.getId()
        dict['title'] = obj.title
        dict['depth'] = depth
        dict['order'] = order
        dict['parent'] = parentpath # XXX not VHM-resistant
        ppath = obj.getPhysicalPath()
        path = '/'.join(ppath)
        dict['path'] = path
        rurl = getToolByName(self, 'portal_url').getRelativeUrl(obj)
        if rurl: 
            rurl = '/'+rurl
        dict['rurl'] = rurl # url relative to portal
        if depth == -1: # skipped root may have no getLevelKey method
            levelkey = ''
        else:
            obj.get_level_additional_info(dict)
            try:
                levelkey = obj.getLevelKey()
            except AttributeError:
                raise KeyError, 'The hierarchy root path is probably incorrect'
        dict['levelkey'] = levelkey
        # children = obj.listSubtopics()
        children = obj.objectValues([self.HierarchyLevelType])
        childrenlist = []
        for child in children:
            order = order+1
            childpath = self._recursive_build(child, depth+1, path, hierdict, order)
            childrenlist.append(childpath)
        dict['children'] = tuple(childrenlist)
        hierdict[path] = dict
        return path

    security.declarePrivate('_recursive_build_list')
    def _recursive_build_list(self, path, hierlist, hierdict):
        # build the full ordered list, depth first
        for childpath in hierdict[path]['children']:
            hierlist.append(childpath)
            self._recursive_build_list(childpath, hierlist, hierdict)

    security.declarePrivate('_get_hierarchy')
    def _get_hierarchy(self):
        # get hierarchy dictionary information
        root = self.getRootObject()
        hierdict = {}
        hierkeydict = {}
        if self.skiproot:
            depth = -1
        else:
            depth = 0
        rootpath = self._recursive_build(root, depth, None, hierdict, 1)
        # build reverse keys dictionnary
        hierkeys = {}
        for path in hierdict.keys():
            if self.skiproot and path == rootpath:
                continue
            info = hierdict[path]
            levelkey = info['levelkey']
            hierkeys[levelkey] = path
            hierkeydict[levelkey] = info
        # build ordered list
        if self.skiproot:
            hierlist = []
        else:
            hierlist = [rootpath]
        self._recursive_build_list(rootpath, hierlist, hierdict)
        return hierlist, hierdict, hierkeys, hierkeydict

    #
    # ZMI
    #

    manage_options = (SimpleItemWithProperties.manage_options[:1] +
                      (
                       {'label': 'Contents',
                        'action': 'manage_hierarchyView'},
                       ) +
                      SimpleItemWithProperties.manage_options[1:])

    security.declareProtected(ManagePortal, 'manage_hierarchyView')
    manage_hierarchyView = DTMLFile('dtml/manage_reviewedHierarchyView', globals())

    security.declarePublic(RebuildHierarchy, 'manage_rebuildHierarchy')
    def manage_rebuildHierarchy(self, REQUEST=None):
        """Rebuild this hierarchy."""
        #root = self.getRootObject()
        self.rebuildHierarchy()
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url()
                + '/manage_hierarchyView')

    security.declareProtected(ManagePortal, 'manage_hierarchyView')
    manage_hierarchyView = DTMLFile('dtml/manage_hierarchyView', globals())

    #
    # Whole hierarchy methods
    #

    security.declareProtected(RebuildHierarchy, 'rebuildHierarchy')
    def rebuildHierarchy(self):
        """Rebuild this hierarchy."""
        hierlist, hierdict, hierkeys, hierkeydict = self._get_hierarchy()
        self.hierlist = hierlist # list of info
        self.hierdict = hierdict # dict of info by path
        self.hierkeys = hierkeys # list of keys
        self.hierkeydict = hierkeydict # dict of info by key

    security.declareProtected(AccessContentsInformation, 'getHierarchyLevelKeys')
    def getHierarchyLevelKeys(self):
        """Gets a list of hierarchy level keys."""
        return self.hierkeys.keys()

    security.declareProtected(AccessContentsInformation, 'getHierarchyList')
    def getHierarchyList(self):
        """Get a list with the hierarchy information."""
        res = []
        hierdict = self.hierdict
        for path in self.hierlist:
            d = hierdict[path].copy()
            res.append(d)
        return res

    security.declareProtected(AccessContentsInformation, 'getHierarchyDict')
    def getHierarchyDict(self):
        """Get a dict by path with the hierarchy information."""
        return copy.deepcopy(self.hierdict)

    security.declareProtected(AccessContentsInformation, 'getHierarchyKeyDict')
    def getHierarchyKeyDict(self):
        """Gets a dict by key with the hierarchy information."""
        return copy.deepcopy(self.hierkeydict)

    security.declareProtected(AccessContentsInformation, 'buildRecursiveQuery')
    def buildRecursiveQuery(self):
        """Build the query for a level and all its sublevels."""
        pass
        # XXX

    security.declareProtected(AccessContentsInformation, 'getLevelByKey')
    def getLevelByKey(self, levelkey):
        """Find the level object with a given key."""
        path = self.hierkeys.get(levelkey, None)
        if path is None:
            return None
        return self.restrictedTraverse(path)


InitializeClass(BaseHierarchy)

