# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Author: Florent Guillaume <mailto:fg@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: ReviewedPublication.py,v 1.8 2003/05/21 14:39:22 guillaum Exp $

"""
  Reviewed Publication mixin.
"""

__version__ = '$Revision: 1.8 $'[11:-2]

from zLOG import LOG, DEBUG

from Globals import InitializeClass
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo, getSecurityManager

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import \
     AccessContentsInformation, View, ModifyPortalContent, AddPortalContent

from publication_utils import modifyRolesForPermission


AnonymousRole = 'Anonymous'
ManagerRole = 'Manager'
ReaderRole = 'Reader'
ReviewerRole = 'Reviewer'
WGManagerRole = 'WorkgroupManager'
WGMemberRole = 'WorkgroupMember'
WGVisitorRole = 'WorkgroupVisitor'


class ReviewedPublication:
    """Reviewed Publication mixin.

    This gives documents the ability to change state in the
    workflow depending on their publication status.
    """

    security = ClassSecurityInfo()

    #
    # Workflow-related methods
    #

    # XXX check security assertions


    # called also by ReviewedHierarchyLevel
    security.declarePublic('invalidateWorkflowInfoCache')
    def invalidateWorkflowInfoCache(self):
        """Invalidates the info cache."""
        try:
            self.REQUEST.other['publication_workflow_info'] = None
        except:
            pass

    # called also by workflow
    security.declarePublic('getPublicationWorkflowInfoFor')
    def getPublicationWorkflowInfoFor(self, hiername):
        """Gets information about workflow publication."""
        path = self.getPhysicalPath()
        try:
            # XXX check hiername too
            info = self.REQUEST.other.get('publication_workflow_info', None)
            if info['path'] != path:
                info = None
        except:
            info = None
        if info is not None:
            return info

        sm = getSecurityManager()
        user = sm.getUser()
        myid = user.getUserName()
        try:
            mygroups = user.getGroups()
        except AttributeError:
            mygroups = ()
        myids = [myid] + ['group:' + id for id in mygroups]

        roles = user.getRolesInContext(self)
        canchange = WGManagerRole in roles or WGMemberRole in roles

        catalog = getToolByName(self, 'portal_catalog')
        htool = getToolByName(self, 'portal_hierarchies')
        hier = htool.getHierarchyById(hiername)

        # all: all keys for the hierarchy
        all = hier.getHierarchyLevelKeys()
        # (using a list is not the optimally fastest method but it should
        # be okay...)

        # pub: all published keys
        pub = getattr(self, '_public_categories', [])
        # pend: all pending keys
        pend = getattr(self, '_pending_categories', [])

        # review: levels for which the user is reviewer
        levelbrains = catalog.searchResults(meta_type=hier.HierarchyLevelType,
                                            LevelReviewers=myids)
        review = [brain.levelkey for brain in levelbrains 
                                 if brain.levelkey in all]
        if 'Manager' in roles:
            review = all

        # read: levels the user is allowed to see
        levelbrains = (catalog.searchResults(meta_type=hier.HierarchyLevelType,
                                             LevelReaders=myids) +
                       catalog.searchResults(meta_type=hier.HierarchyLevelType,
                                             LevelIsPublic=1))
        read = [brain.levelkey for brain in levelbrains 
                               if brain.levelkey in all]
        for key in review:
            if key not in read:
                read.append(key)

        #
        # Variables consulted by guard and action display
        #

        # depublish: review inter pub
        depublish = [key for key in review if key in pub]

        # accept: review inter pend
        accept = [key for key in review if key in pend]

        # publish: review-(pub+pend)
        publish = [key for key in review if not (key in pub or key in pend)]

        # reject: review inter pend
        reject = accept

        # submit: read-(review+pub+pend)
        if canchange:
            submit = [key for key in read 
                          if not (key in review or key in pub or key in pend)]
        else:
            submit = []

        # retract: read inter (pub+pend-review)
        if canchange:
            retract = [key for key in read 
                           if (key in pub or key in pend) and key not in review]
        else:
            retract = []

        pubnext = publish+submit
        pubprev = retract

        info = {
            'path': path,
            'all': all,
            'review': review,
            'read': read,
            'pend': pend,
            'pub': pub,
            #
            'creation': [],
            'publish': publish,
            'accept': accept,
            'submit': submit,
            'reject': reject,
            'retract': retract,
            'depublish': depublish,
            'pubnext': pubnext,
            'pubprev': pubprev,
            'refresh': [],
        }

        try:
            self.REQUEST.other['publication_workflow_info'] = info
        except:
            pass
        return info

    # called by ReviewedHierarchyLevel upon delete
    security.declarePrivate('removeCategory')
    def removeCategory(self, key, hiername):
        """A level is deleted, so the key disappears."""
        changed = 0
        pub = getattr(self, '_public_categories', ())
        if key in pub:
            while key in pub: pub.remove(key)
            self._public_categories = pub
            changed = 1
        pend = getattr(self, '_pending_categories', ())
        if key in pend:
            while key in pend: pend.remove(key)
            self._pending_categories = pend
            changed = 1
        if changed:
            self.setWorkflowStatePermissions(hiername)

    # called by workflow
    security.declarePublic('updateCategories')
    def updateCategories(self, state_change, keys, hiername):
        """Updates the public and pending categories after a transition."""
        transition = state_change.transition.getId()
        if transition not in ('creation', 'publish', 'accept', 'depublish',
                              'submit', 'reject', 'retract', 'refresh'):
            raise ValueError, 'Invalid transition %s' % transition

        info = self.getPublicationWorkflowInfoFor(hiername)
        okkeys = info.get(transition, ())
        dokeys = [key for key in keys if key in okkeys]

        # PublicCategories
        old = getattr(self, '_public_categories', [])
        if transition in ('publish', 'accept'):
            new = list(old)
            for key in dokeys:
                if key not in old:
                    new.append(key)
        elif transition in ('reject', 'depublish', 'retract'):
            new = [key for key in old if key not in dokeys]
        else: # transition in ('submit', 'retract', 'creation', 'refresh')
            new = old
        self._public_categories = new

        # PendingCategories
        old = getattr(self, '_pending_categories', [])
        if transition == 'submit':
            new = list(old)
            for key in dokeys:
                if key not in old:
                    new.append(key)
        elif transition in ('publish', 'accept', 'reject', 'retract'):
            new = [key for key in old if key not in dokeys]
        else: # transition in ('creation', 'refresh')
            new = old
        self._pending_categories = new

        self.invalidateWorkflowInfoCache()


    # called by workflow
    security.declarePublic('getWorkflowVariable')
    def getWorkflowVariable(self, var, hiername):
        """Gets new workflow variables."""
        htool = getToolByName(self, 'portal_hierarchies')
        hier = htool.getHierarchyById(hiername)
        info = self.getPublicationWorkflowInfoFor(hiername)
        if var == hier.criterion: # PublicCategories
            return info['pub']
        if var == hier.criterion_pending: # PendingCategories
            return info['pend']
        if var == 'review_state':
            if info['pub']:
                return 'published'
            elif info['pend']:
                return 'pending'
            else:
                return 'private'
        raise ValueError, 'Invalid variable %s' % var


    security.declarePrivate('_gather_readers_reviewers')
    def _gather_readers_reviewers(self, pend, pub, hiername):
        """Returns a list of all potential readers and reviewers for the
        object's levels: (reviewers, readers, public)

        public = true means allow public access.
        """
        htool = getToolByName(self, 'portal_hierarchies')
        hier = htool.getHierarchyById(hiername)
        # XXX Readers are acquired in the level...
        reviewers = {}
        readers = {}
        public = 0
        for key in pend + pub:
            level = hier.getLevelByKey(key)
            if level is None:
                continue # old key
            for id in level.LevelReviewers():
                reviewers[id] = None
            if key in pub:
                if level.LevelIsPublic():
                    public = 1
                for id in level.LevelReaders():
                    readers[id] = None
        if public:
            readers = {}
        reviewers = reviewers.keys()
        readers = readers.keys()
        return (reviewers, readers, public)


    # called by workflow scripts, and by ReviewedHierarchyLevel
    security.declarePublic('setWorkflowStatePermissions')
    def setWorkflowStatePermissions(self, hiername):
        """Sets new permissions/roles after this object changed levels
        or after one of its levels changed of reviewers/readers.
        """
        wftool = getToolByName(self, 'portal_workflow')
        htool = getToolByName(self, 'portal_hierarchies')
        hier = htool.getHierarchyById(hiername)
        pub = wftool.getInfoFor(self, hier.criterion)
        pend = wftool.getInfoFor(self, hier.criterion_pending)

        #
        # Remove local roles Reader and Reviewer
        #
        # User local roles
        lroles = self.get_local_roles()
        for id, roles in lroles:
            roles = list(roles)
            changed = 0
            if ReaderRole in roles:
                roles.remove(ReaderRole)
                changed = 1
            if ReviewerRole in roles:
                roles.remove(ReviewerRole)
                changed = 1
            if changed:
                if roles:
                    self.manage_setLocalRoles(id, roles)
                else:
                    self.manage_delLocalRoles((id,))
        # Group local roles
        try:
            lgroups = self.get_local_group_roles()
        except AttributeError:
            lgroups = ()
        for id, roles in lgroups:
            roles = list(roles)
            changed = 0
            if ReaderRole in roles:
                roles.remove(ReaderRole)
                changed = 1
            if ReviewerRole in roles:
                roles.remove(ReviewerRole)
                changed = 1
            if changed:
                if roles:
                    self.manage_setLocalGroupRoles(id, roles)
                else:
                    self.manage_delLocalGroupRoles((id,))

        #
        # Set adequate local roles
        #
        (reviewers, readers, public) = self._gather_readers_reviewers(pend, pub, hiername)
        for (ids, role) in ((reviewers, ReviewerRole), (readers, ReaderRole)):
            for id in ids:
                if not id.startswith('group:'):
                    self.manage_addLocalRoles(id, (role,))
                else:
                    id = id[len('group:'):]
                    try:
                        self.manage_addLocalGroupRoles(id, (role,))
                    except AttributeError:
                        pass

        #
        # Set permissions
        #
        WGR = (WGManagerRole, WGMemberRole, WGVisitorRole)
        WGRW = (WGManagerRole, WGMemberRole)
        get_modifier_roles = getattr(aq_base(self), 'get_modifier_roles', None)
        if get_modifier_roles is not None:
            modify = get_modifier_roles()
        else:
            modify = None
        if pub or pend:
            if modify is None:
                LOG('ReviewedPublication', DEBUG,
                  "Can't get modifier roles from other WF")
                modify = (ManagerRole,)
            view = (ManagerRole, ReaderRole, ReviewerRole) + WGR
            if public:
                view = view + (AnonymousRole,)
        else:
            if modify is None:
                LOG('ReviewedPublication', DEBUG,
                  "Can't get modifier roles from other WF")
                modify = (ManagerRole,) + WGRW
            view = (ManagerRole,) + WGR

        modifyRolesForPermission(self, ModifyPortalContent, modify)
        modifyRolesForPermission(self, View, view)
        modifyRolesForPermission(self, AccessContentsInformation, view)

        # reindex security
        self.reindexObjectSecurity()



##     # called by workflow scripts, and by ReviewedHierarchyLevel
##     security.declarePublic('setWorkflowStatePermissions')
##     def setWorkflowStatePermissions(self, hiername):
##         """
##         Sets new permissions/roles after a workflow transition,
##         or after a level changed reviewers/readers.
##         """
##         wftool = getToolByName(self, 'portal_workflow')
##         htool = getToolByName(self, 'portal_hierarchies')
##         hier = htool.getHierarchyById(hiername)
##         pub = wftool.getInfoFor(self, hier.criterion)
##         pend = wftool.getInfoFor(self, hier.criterion_pending)

##         #
##         # Remove local role Reader
##         #
##         # User local roles
##         lroles = self.get_local_roles()
##         for id, roles in lroles:
##             if ReaderRole in roles:
##                 roles = list(roles)
##                 roles.remove(ReaderRole)
##                 if roles:
##                     self.manage_setLocalRoles(id, roles)
##                 else:
##                     self.manage_delLocalRoles((id,))
##         # Group local roles
##         try:
##             lgroups = self.get_local_group_roles()
##         except:
##             lgroups = ()
##         for id, roles in lgroups:
##             if ReaderRole in roles:
##                 roles = list(roles)
##                 roles.remove(ReaderRole)
##                 if roles:
##                     self.manage_setLocalGroupRoles(id, roles)
##                 else:
##                     self.manage_delLocalGroupRoles((id,))

##         #
##         # Set adequate local roles
##         #
##         if pub or pend:
##             # Set public view for reviewers and readers

##             # Disacquire Modify: (Manager)
##             modifyRolesForPermission(self, ModifyPortalContent,
##                                      (ManagerRole,))
##             # Acquire, ACI & View: [Reader, Manager] or [Anonymous, Manager]
##             readers = self._gather_readers(pend, pub, hiername)
##             readrole = readers and ReaderRole or AnonymousRole
##             modifyRolesForPermission(self, AccessContentsInformation,
##                                      [ManagerRole, readrole])
##             modifyRolesForPermission(self, View,
##                                      [ManagerRole, readrole])
##             if readers:
##                 # set local role Reader for the readers
##                 ureaders = filter(lambda id: id[:6] != 'group:', readers)
##                 greaders = filter(lambda id: id[:6] == 'group:', readers)
##                 greaders = map(lambda id: id[6:], greaders)
##                 for id in ureaders:
##                     self.manage_addLocalRoles(id, (ReaderRole,))
##                 for id in greaders:
##                     self.manage_addLocalGroupRoles(id, (ReaderRole,))

##         else:
##             # Set private permissions
##             modifyRolesForPermission(self, AccessContentsInformation,
##                                      (ManagerRole, WGManagerRole, WGMemberRole, WGVisitorRole))
##             modifyRolesForPermission(self, View,
##                                      (ManagerRole, WGManagerRole, WGMemberRole, WGVisitorRole))
##             modifyRolesForPermission(self, ModifyPortalContent,
##                                      (ManagerRole, WGManagerRole, WGMemberRole))

##         self.reindexObjectSecurityRecursive() # XXX recursive ?


InitializeClass(ReviewedPublication)

