# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: PortalBoxesTool.py,v 1.18 2003/04/06 09:55:52 fermigier Exp $

"""
  PortalBoxesTool
"""

__version__='$Revision: 1.18 $'[11:-2]

from zLOG import LOG, DEBUG

from DateTime import DateTime
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo, getSecurityManager, Unauthorized
from AccessControl.PermissionRole import rolesForPermissionOn
from Acquisition import aq_inner, aq_parent, aq_chain, aq_base

from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem


from Products.CMFCore.CMFCorePermissions import setDefaultRoles, \
     View, AccessContentsInformation, ManagePortal
from Products.CMFCore.utils import UniqueObject, getToolByName, \
     _checkPermission


def cmporderbox(a, b):
    return cmp(a.xpos, b.xpos) or cmp(a.ypos, b.ypos)


class PortalBoxesTool(UniqueObject, SimpleItem):
    """
    Portal Boxes Tool.
    """
    id = 'portal_boxes'
    meta_type = 'Portal Boxes Tool'

    security = ClassSecurityInfo()

    manage_options = (
        ({'label': "Overview", 'action': 'manage_overview',},) +
        SimpleItem.manage_options
        )

    #
    # ZMI
    #

    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = DTMLFile('zmi/explainPortalBoxesTool', globals())

    #
    # Public API
    #

    security.declarePublic('getBoxTypes')
    def getBoxTypes(self, container=None):
        """Gets all Box types allowed in container.

        Returns a sequence of (id, title).
        """
        ttool = getToolByName(self, 'portal_types')
        tilist = ttool.listTypeInfo(container)
        res = []
        for ti in tilist:
            if ti.getActionById('isportalbox', None):
                d = {
                    'id': ti.getId(),
                    'title': ti.Title(),
                    }
                res.append(d)
        return res

    security.declarePublic('getBoxesWithIds')
    def getBoxesWithIds(self, xpos=None, alsoclosed=0, perso=0, permission = 'View'):
        """Gets all the boxes of a user

        Returns a dict { boxid: box }
        """
        container = self.getBoxesContainer(perso=perso)
        if container is None:
            return {}
        # get all boxes
        boxes = container.objectItems()
        mapping = {}
        if permission is not None:
            securityManager = getSecurityManager()
        for boxid, box in boxes:
            if permission is not None and not securityManager.checkPermission(permission, box):
                continue
            if xpos is not None and xpos != box.xpos:
                continue
            if not alsoclosed and box.closed:
                continue
            mapping[boxid] = box
        return mapping

    security.declarePublic('getBoxes')
    def getBoxes(self, xpos=None, alsoclosed=0, all=1, perso=1, permission = 'View'):
        """Gets all the boxes of a user sorted according to xpos and ypos."""
        boxes = []
        if all:
            this_boxes = self.getBoxesWithIds(
                xpos=xpos, alsoclosed=alsoclosed,
                perso=0, permission=permission).values()
            this_boxes.sort(cmporderbox)
            boxes.extend(this_boxes)
        if perso:
            this_boxes = self.getBoxesWithIds(
                xpos=xpos, alsoclosed=alsoclosed,
                perso=1, permission=permission).values()
            this_boxes.sort(cmporderbox)
            boxes.extend(this_boxes)

        return boxes

    security.declarePublic('getBoxesContainer')
    def getBoxesContainer(self, create=1, perso=0):
        """Gets the user's (or global) boxes container, maybe creates it.

        Returns None if the user can't have private boxes.
        """
        if perso:
            ptool = getToolByName(self, 'portal_preferences')
            prefs = ptool.getUserPreferences(create=create, maybe_global=0)
        else:
            utool = getToolByName(self, 'portal_url')
            prefs = utool.getPortalObject().Global.Preferences

        if prefs is None:
            return None
            
        container = getattr(prefs, 'boxes_container', None)
        if container is None and create:
            prefs.invokeFactory('Folder', 'boxes_container')
            container = prefs.boxes_container

        return container

    security.declarePublic('setBoxState')
    def setBoxState(self, boxid, state, perso=0):
        """Sets the box state, in the private box container."""
        # XXX should we do any security check ?
        if not perso:
            box = self.getBoxForId(boxid, perso=perso, try_perso=0)
            if box is None:
                perso = 1
        if perso:
            box = self.getBoxForId(boxid, perso=perso)
        response = None
        if box is not None:
            valid_states = self.get_valid_states()
            valid_keys = valid_states.keys()
            for key, value in state.items():
                if key in valid_keys:
                    can_change_key = getattr(box, 'can_' + key)
                    if callable(can_change_key):
                        can_change_key = can_change_key()
                    if can_change_key:
                        if valid_states[key] == 'bool':
                            if value:
                                value='on'
                            else:
                                value=''
                        if valid_states[key] == 'int':
                            value = int(value)
                        if key == 'style':
                            can_minimized = 0
                            meth = getattr(self, 'boxes_styles_get', None)
                            if meth is not None:
                                styles = self.boxes_styles_get()
                                for style in styles:
                                    if style['id'] == value:
                                        can_minimized = style.get('can_minimized')
                                        break
                            box._can_minimized = can_minimized
                        if key == 'minimized':
                            cookie_name = '%s_%s' % (box.cps_prefId(), key)
                            if response is None:
                                response = self.REQUEST.RESPONSE
                            response.setCookie(cookie_name, value, expires= (DateTime()+365).toZone('GMT').rfc822())
                        else:
                            setattr(box, key, value)

    security.declarePublic('getBoxForId')
    def getBoxForId(self, boxid, perso=0, try_perso=1):
        if not perso:
            container = self.getBoxesContainer(perso=0)
            if boxid in container.objectIds():
                return getattr(container, boxid)
        if perso or try_perso:
            container = self.getBoxesContainer(perso=1)
            if boxid in container.objectIds():
                return getattr(container, boxid)
        return None

    security.declarePublic('delBox')
    def delBox(self, boxid, perso=0, try_perso=1):
        if not perso:
            container = self.getBoxesContainer(perso=0)
            if boxid not in container.objectIds():
                container = None
        if perso or (container is None and try_perso):
            container = self.getBoxesContainer(perso=1)
            if boxid not in container.objectIds():
                container = None

        if container is None:
            return

        mtool = getToolByName(self, 'portal_membership')
        if not mtool.checkPermission('Delete objects', container):
            raise Unauthorized
        container._delObject(boxid)

    #
    # Private
    #

    security.declarePrivate('getDefaultState')
    def getDefaultState(self):
        """Returns the default state."""
        return self._default_state.copy()

    _default_state = {
        'closed': 0,
        'minimized': 0,
        'xpos': 1,
        'ypos': 0,
        'style': '',
        }

    security.declarePublic('get_valid_states')
    def get_valid_states(self):
        return {
                'closed': 'bool',
                'minimized': 'bool',
                'xpos': 'int',
                'ypos': 'int',
                'style': 'text'
               }


InitializeClass(PortalBoxesTool)

